# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2013, Barna Keresztes <kbarni@kbarni.net>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gtk # pylint: disable=E0611

from uberwriter_lib.helpers import get_builder

import os
import subprocess
import locale

import logging
logger = logging.getLogger('uberwriter')

import gettext
from gettext import gettext as _
gettext.textdomain('uberwriter')

class UberwriterHtmlExportDialog(Gtk.Dialog):
    __gtype_name__ = "UberwriterHtmlExportDialog"

    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated UberwriterHtmlExportDialog object.
        """
        builder = get_builder('UberwriterHtmlExportDialog')
        new_object = builder.get_object('uberwriter_html_export_dialog')
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        """Called when we're finished initializing.

        finish_initalizing should be called after parsing the ui definition
        and creating a UberwriterHtmlExportDialog object with it in order to
        finish initializing the start of the new UberwriterHtmlExportDialog
        instance.
        """
        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self)

    def advanced_export(self, text = ""):
        filechooser = Gtk.FileChooserDialog(
            "Export HTML",
            self,
            Gtk.FileChooserAction.SAVE,
            (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
             Gtk.STOCK_SAVE, Gtk.ResponseType.OK)
            )

        filechooser.set_do_overwrite_confirmation(True)
        
        response = filechooser.run()
        if response == Gtk.ResponseType.OK:
            filename = filechooser.get_filename()
            filechooser.destroy()
        else: 
            filechooser.destroy()
            return 
                
        output_dir = os.path.abspath(os.path.join(filename, os.path.pardir))
        
        basename = os.path.basename(filename)

        args = ['pandoc', '--from=markdown']
        
        to = "--to=html"
        
        if basename.endswith(".html") or basename.endswith(".htm"):
            output_file = "--output=%s" % basename
        else:
            output_file = "--output=%s.%s" % (basename, "html")
        #--toc,--smart,--normalize, --standalone, number_sections
            
        if self.builder.get_object("smart").get_active():
            args.append('--smart')
            args.append('--normalize')
        
        args.append("--mathjax")
        args.append("--self-contained")

        if self.builder.get_object("html5").get_active():
            to = "--to=%s" % "html5"

        css_uri = self.builder.get_object("css_filechooser").get_uri()
        if css_uri:
            if css_uri.startswith("file://"):
                css_uri = css_uri[7:]
            args.append("--css=%s" % css_uri)
                                    
        args.append(to)     
        
        args.append(output_file)

        logger.info(args)

        p = subprocess.Popen(args, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=output_dir)

        output = p.communicate(text)
        
        if output[1]!= b"" :
            logger.info("Error during exporting");
            md = Gtk.MessageDialog(self, 0, Gtk.MessageType.ERROR, 
                    Gtk.ButtonsType.CLOSE, "Error during exporting")
            md.format_secondary_text(output[1].decode("utf-8"))
            md.run()
            md.destroy()
        
        return filename
        
    def on_btn_ok_clicked(self, widget, data=None):
        """The user has elected to save the changes.

        Called before the dialog returns Gtk.ResponseType.OK from run().
        """
        pass

    def on_btn_cancel_clicked(self, widget, data=None):
        """The user has elected cancel changes.

        Called before the dialog returns Gtk.ResponseType.CANCEL for run()
        """
        pass


if __name__ == "__main__":
    dialog = UberwriterHtmlExportDialog()
    dialog.show()
    Gtk.main()
