# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2013, Barna Keresztes <kbarni@kbarni.net>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gtk # pylint: disable=E0611

from uberwriter_lib.helpers import get_builder

import os
import subprocess
import locale

import logging
logger = logging.getLogger('uberwriter')

import gettext
from gettext import gettext as _
gettext.textdomain('uberwriter')

class UberwriterLatexExportDialog(Gtk.Dialog):
    __gtype_name__ = "UberwriterLatexExportDialog"

    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated UberwriterLatexExportDialog object.
        """
        builder = get_builder('UberwriterLatexExportDialog')
        new_object = builder.get_object('uberwriter_latex_export_dialog')
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        """Called when we're finished initializing.

        finish_initalizing should be called after parsing the ui definition
        and creating a UberwriterLatexExportDialog object with it in order to
        finish initializing the start of the new UberwriterLatexExportDialog
        instance.
        """

        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self)

        self.builder.get_object("doctype_combo").set_active(0)
        self.builder.get_object("pagesize_combo").set_active(0)
        self.builder.get_object("fontsize_combo").set_active(0)
        self.builder.get_object("orientation_combo").set_active(0)
        
    def advanced_export(self, text = "", filetype="",fileformat="",extension=""):
        filechooser = Gtk.FileChooserDialog(
            "Export %s" % filetype,
            self,
            Gtk.FileChooserAction.SAVE,
            (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
             Gtk.STOCK_SAVE, Gtk.ResponseType.OK)
            )

        filechooser.set_do_overwrite_confirmation(True)
        
        response = filechooser.run()
        if response == Gtk.ResponseType.OK:
            filename = filechooser.get_filename()
            filechooser.destroy()
        else: 
            filechooser.destroy()
            return 
                
        output_dir = os.path.abspath(os.path.join(filename, os.path.pardir))
        
        basename = os.path.basename(filename)

        args = ['pandoc', '--from=markdown']
        
        to = "--to=%s" % fileformat
        
        if basename.endswith("." + extension):
            output_file = "--output=%s" % basename
        else:
            output_file = "--output=%s.%s" % (basename, extension)
        #--toc,--smart,--normalize, --standalone, number_sections
        if self.builder.get_object("toc").get_active():
            args.append('--toc')
        #if self.builder.get_object("normalize").get_active(): # combined with the smart option
            
        if self.builder.get_object("smart").get_active():
            args.append('--smart')
            args.append('--normalize')
        if self.builder.get_object("standalone").get_active():
            args.append("--standalone")
        if self.builder.get_object("number_sections").get_active():
            args.append("--number-sections")
        #--variables: document type, page site, font size, page orientation, 
        args.append("--variable=documentclass:{0}".format(self.builder.get_object("doctype_combo").get_active_text()))
        orientation=""
        titlepage = ""
        if self.builder.get_object("orientation_combo").get_active_text()=="Landscape":
            orientation=",landscape"
        if self.builder.get_object("titlepage").get_active():
            titlepage=",titlepage"
        args.append("--variable=fontsize:{0},{1}{2}{3}".format(self.builder.get_object("fontsize_combo").get_active_text(),self.builder.get_object("pagesize_combo").get_active_text(),orientation,titlepage))
        #...--latex-engine=*pdflatex*|luatex|xelatex
        #TODO: implement the latex-engine selection
        
        #--bibliography=[file]
        bib_uri = self.builder.get_object("bib_filechooser").get_uri()
        if bib_uri:
            if bib_uri.startswith("file://"):
                bib_uri = bib_uri[7:]
            args.append("--bibliography=%s" % bib_uri)
        
        if extension!="pdf":        
            args.append(to)     
        
        args.append(output_file)

        logger.info(args)

        p = subprocess.Popen(args, stdin=subprocess.PIPE, 
            stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=output_dir)

        output = p.communicate(text)

        if output[1]!= b"" :
            logger.info("Error during exporting");
            md = Gtk.MessageDialog(self, 0, Gtk.MessageType.ERROR, 
                    Gtk.ButtonsType.CLOSE, "Error during exporting")
            md.format_secondary_text(output[1].decode("utf-8"))
            md.run()
            md.destroy()
        
        return filename
        
    def on_btn_ok_clicked(self, widget, data=None):
        """The user has elected to save the changes.

        Called before the dialog returns Gtk.ResponseType.OK from run().
        """
        pass

    def on_btn_cancel_clicked(self, widget, data=None):
        """The user has elected cancel changes.

        Called before the dialog returns Gtk.ResponseType.CANCEL for run()
        """
        pass


if __name__ == "__main__":
    dialog = UberwriterLatexExportDialog()
    dialog.show()
    Gtk.main()
